/** 
 *   Site testing for SKA candidate.
 *
 *   (c) Copyright 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO:
 *
 *
 **/

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include "my_stdio.h"
#include "observ_table.h"
#include "analyzer.h"
#include "toska.h"


/* fifo */
static char *gpc_name = NULL;
static FILE *fgpc;
static pid_t gnuplot_pid;

static pid_t my_exec(char *command)
{
	pid_t pid;

	if (command == 0)
		return 1;
	pid = fork();
	if (pid == -1)
		return -1;
	if (pid == 0) {
		char *argv[4];
		argv[0] = "sh";
		argv[1] = "-c";
		argv[2] = command;
		argv[3] = 0;
		if( execve("/bin/sh", argv, environ) == -1){
			perror("execve");
			exit(1);
		}
		exit(127);
	}
	return pid;
}


static char *gnuplot_command[] = {
//	"cat -",
//	"",
	"gnuplot",
	"-geometry 640x480-4+0",
	"-pointsize 4",
	""
};


static char *plot_setup[] = {
	"set nokey",
	"set title \"-- SKA Survey (IAR) --\"",
	"set ylabel \"[dbm]\"",
	"set xlabel \"Frequency [GHz]\"",
	"set grid",
	""
};


int open_plot(void)
{
	char command[512];
	int i;

	if (gpc_name == NULL) {
		gpc_name = (char *) malloc(_POSIX_PATH_MAX *sizeof(char));
		sprintf( gpc_name, "/tmp/gpc_fifo");
		unlink(gpc_name);
		if( mkfifo(gpc_name, 0600) == -1) {
			perror("open_plot->mkfifo");
			return -1;
		}
		*(command) = 0;
		for(i=0; (char) *gnuplot_command[i] != 0; i++)
			sprintf(command+strlen(command),"%s ",
				gnuplot_command[i]);
		sprintf(command+strlen(command),
			"-title \"%s: plot\" ", name_definition);
		sprintf(command+strlen(command),"< %s", gpc_name );
		PVERB(PINFO,"gnuplot command:\n%s\n", command);
		gnuplot_pid = my_exec( command );
		if((fgpc = fopen(gpc_name, "w")) == NULL) {
			perror("fopen");
			return -1;
		}
		for(i=0; (char) *plot_setup[i] != 0; i++)
			fprintf(fgpc,"%s\n", plot_setup[i]);
	}

	return gnuplot_pid;
}


int close_plot(void)
{
	int status;

	fprintf(fgpc,"q\n");
	fflush(fgpc);
	if (waitpid(gnuplot_pid, &status, 0) == -1) {
		PVERB(PERR,"close_plot status: %d\n", status);
	}
	fclose(fgpc);
	unlink(gpc_name);
	free(gpc_name);
	gpc_name = NULL;
	
	return 0;
}


int plot_observ(struct obs_setup *s)
{
	fprintf(fgpc,"set yrange [%f:%f]\n", AN_REF - 80.0, AN_REF);
	fprintf(fgpc,"set xrange [%f:%f]\n", 
		s->obs->start_freq, s->obs->stop_freq);

	fprintf(fgpc, "set label 1 \"File: %s\" at graph 0.05,0.95\n", 
		s->file);
	fprintf(fgpc,
		"set label 2 \"Mode: %d, Azt: %3d%c, Rep: %d/%d\" at graph 0.05,0.9\n",
		s->mode,  s->azimuth, s->polarization, s->current_rep, 
		s->obs->repetitions);
	fprintf(fgpc, "set label 3 \"Sweep: %d\" at graph 0.05,0.85\n", 
		s->obs->sweep_obs);
	fprintf(fgpc, "set label 4 \"RBW: %d KHz\" at graph 0.05,0.80\n", 
		s->obs->RBW);

	fprintf(fgpc,"plot \"%s\" u ($0 * %f + %f):1 w l lw 1\n", 
		s->file, 
		(s->obs->stop_freq - s->obs->start_freq) / (double) s->naxis1,
		s->obs->start_freq);

	fflush(fgpc);

	return 0;
}

